/* Mlib version 2.02 by Michael McElroy
lord_nightrose@yahoo.com
C Source File
Created 4/24/01; 9:30:30 PM
Mlib is a library of advanced TI-89 functions, written in C. 
Thanks to Thomas Nussbaumer and Francois Leiber for their inspiration and help. */

#define RETURN_VALUE ml // Redirect Return Value

#define OPTIMIZE_ROM_CALLS    // Use ROM Call Optimization

// #define SAVE_SCREEN        // Save/Restore LCD Contents

#include <tigcclib.h>         // Include All Header Files

short _ti89;                  // Produce .89Z File

// Main Function
void _main(void) 
{
// setup - create all the variables we'll need
  unsigned long len; // used by 'size' function
  int mode, numofarg, x, y; // used by many functions for mode settings
  ESI arg; // pointer to argument list
  ESI fileptr;
  SYM_ENTRY *vatentry, *argument, *FPtr; // symbolic entries for finding file & folder lists
  char *function, *text; // function entered, text entered as argument
  InitArgPtr(arg); // set arg to point to the first argument
  push_END_TAG (); // begin the return list

// main function loop; loop until last argument is reached

  while(GetArgType(arg)!=END_TAG) {

//check to see if the current argument is a string. If not, set the function to "" (nothing)

	  numofarg=RemainingArgCnt(arg); // find the number of arguments remaining
    if(GetArgType(arg) == STR_TAG) function = GetStrnArg(arg); else function = "";

// start checking for functions

    if(strcmp(function, "reset") == 0) OSReset();
  
    if(strcmp(function, "err") == 0 && numofarg>=2) {
      if(GetArgType(arg)!=POSINT_TAG) {mode = 0; SkipArg(arg);} else {mode = GetIntArg(arg);}
// throw a user-specified error
          ERD_dialog (mode, 0);
    }

    if(strcmp(function, "freemem") == 0) {
// return free RAM memory
      HeapCompress();
      push_Float_to_nonneg_int(HeapAvail());
    }


// change contrast
    if(strcmp(function, "-") == 0) OSContrastDn();

    if(strcmp(function, "+") == 0) OSContrastUp();
  
// draw a string
    if(strcmp(function, "drawtxt") == 0 && numofarg >= 3) {
      if(GetArgType(arg)!=POSINT_TAG) {x = 0; SkipArg(arg);} else x = GetIntArg(arg);
      if(GetArgType(arg)!=POSINT_TAG) {y = 0; SkipArg(arg);} else y = GetIntArg(arg);
      if(GetArgType(arg)!=STR_TAG) {text = ""; SkipArg(arg);} else text = GetStrnArg(arg);
      printf_xy(x,y,text,A_NORMAL);
    }

    if(strcmp(function, "drawsl") == 0) DrawLine(0, 93, 159, 93, 1);
    
    if(strcmp(function, "font") == 0 && numofarg >= 2) {
      if(GetArgType(arg)!=POSINT_TAG) {mode = 0; SkipArg(arg);} else mode = GetIntArg(arg);
      if(mode>=0 && mode<=2) FontSetSys(mode);
    }

    if(strcmp(function, "clrscr") == 0) ClrScr();

// get a key press from the user
    if(strcmp(function, "getky") == 0) push_Float_to_nonneg_int(ngetchx());
  
    if(strcmp(function, "off") == 0) off();

    if(strcmp(function, "version") == 0) push_ANSI_string("MLib_version_2.02_by_Michael_McElroy");

    if(strcmp(function, "battery") == 0 && numofarg >= 2) {
      if(GetArgType(arg)!=POSINT_TAG) {mode = 0; SkipArg(arg);} else mode = GetIntArg(arg);
      if(mode>=0 && mode<3) ST_batt(mode);
    }
    
    if(strcmp(function, "busy") == 0 && numofarg >= 2) {
      if(GetArgType(arg)!=POSINT_TAG) {mode = 0; SkipArg(arg);} else mode=GetIntArg(arg);
      if(mode>=0 && mode<3) ST_busy(mode);
    }

    if(strcmp(function, "slprec") == 0 && numofarg >= 2) {
      if(GetArgType(arg)!=POSINT_TAG) {mode = 0; SkipArg(arg);} else mode = GetIntArg(arg);
      if(mode>=0 && mode<3) ST_precision(mode);
    }

    if(strcmp(function, "slgraph") == 0 && numofarg >= 2) {
      if(GetArgType(arg) != POSINT_TAG) {mode=0; SkipArg(arg);} else mode = GetIntArg(arg);
      if(mode == 0) ST_graph(mode);
      if(mode >= 1 && mode <= 5) ST_graph(mode+1);
    }
  
    if(strcmp(function, "slfolder") == 0 && numofarg >= 2) {
      if(GetArgType(arg) != STR_TAG) {text = "MAIN"; SkipArg(arg);} else text = GetStrnArg(arg);
      if(strlen(text) <= 8) ST_folder(text);
    }

    if(strcmp(function, "slhelp") == 0 && numofarg >= 2) {
      if(GetArgType(arg) != STR_TAG) {text = "MAIN"; SkipArg(arg);} else text = GetStrnArg(arg);
        ST_helpMsg(text);
    }

    if(strcmp(function, "sleep") == 0 && numofarg >= 2) {
      if(GetArgType(arg) != POSINT_TAG) {mode = 0; SkipArg(arg);} else mode = GetIntArg(arg);
      OSFreeTimer (1);
      OSRegisterTimer (1, mode*20);
      while (!OSTimerExpired (1));
      OSFreeTimer (1);
    }

// put some text in the clipboard
    if(strcmp(function, "cb") == 0 && numofarg >= 2) {
     if(GetArgType(arg) != STR_TAG) {text = ""; SkipArg(arg);} else text = GetStrnArg(arg);
     len = strlen(text);
     CB_replaceTEXT (text,len,0);
    }

// get a list of non-temporary folders
    if(strcmp(function, "folders") == 0) {
      fileptr = top_estack;
      FPtr = SymFindFirst (GetSymstrArg (fileptr), 4);
      while (FPtr)
      {
        push_ANSI_string (FPtr->name);
        FPtr = SymFindNext ();
      }
    }

// get a list of files in a folder
    if(strcmp(function, "files") == 0 && numofarg >= 2) {
      if(GetArgType(arg) != STR_TAG) {FPtr = SymFindFirst ("main", 1); SkipArg(arg);} else FPtr = SymFindFirst (GetSymstrArg (arg), 1);
      while (FPtr) {
        push_ANSI_string (FPtr->name);
        FPtr = SymFindNext ();
      }
    }

// find a file's size
    if(strcmp(function, "size") == 0 && numofarg >= 2) {
      if(GetArgType(arg) != STR_TAG) {argument = "main"; SkipArg(arg);} else argument = GetSymstrArg (arg);
      vatentry = DerefSym(SymFind(argument));
      if(vatentry && vatentry->name) {
        text = HeapDeref(vatentry->handle); // here, text is really the file's name
        len = *(unsigned short*)(text)+2;
        push_Float_to_nonneg_int(len);
      }
      else push_Float_to_nonneg_int(0);
    }
  }
  push_reversed_tail(top_estack); // <--- reverses the list - thanks to Thomas Nussbaumer for his help here!
  push_LIST_TAG ();
}
